@@ -35,7 +35,6 @@
 #include <gtk/gtk.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
-#include <gtk/gtkdrawingarea.h>
 
 #include "gtkmathview_common.h"
 #if GTKMATHVIEW_USES_CUSTOM_READER
@@ -57,6 +56,7 @@ typedef libxml2_MathView MathView;
 typedef gmetadom_MathView MathView;
 #endif
 
+#include "Init.hh"
 #include "Logger.hh"
 #include "Configuration.hh"
 #include "Point.hh"
@@ -64,11 +64,12 @@ typedef gmetadom_MathView MathView;
 #include "MathMLElement.hh"
 #include "MathMLOperatorDictionary.hh"
 #include "MathMLNamespaceContext.hh"
-#include "Gtk_MathGraphicDevice.hh"
 #if ENABLE_BOXML
 #include "BoxMLNamespaceContext.hh"
-#include "Gtk_BoxGraphicDevice.hh"
 #endif // ENABLE_BOXML
+#include "MathGraphicDevice.hh"
+#include "BoxGraphicDevice.hh"
+#include "Gtk_Backend.hh"
 #include "Gtk_RenderingContext.hh"
 #include "Gtk_WrapperArea.hh"
 
@@ -108,14 +109,19 @@ struct _GtkMathViewClass
   GtkMathViewModelSignal select_end;
   GtkMathViewSelectAbortSignal select_abort;
   GtkMathViewModelSignal element_over;
+  GtkMathViewDecorateSignal decorate_under;
+  GtkMathViewDecorateSignal decorate_over;
 
   AbstractLogger* logger;
+  RGBColor defaultForeground;
+  RGBColor defaultBackground;
+  RGBColor defaultSelectForeground;
+  RGBColor defaultSelectBackground;
+  bool defaultT1OpaqueMode;
+  bool defaultT1AntiAliasedMode;
   Configuration* configuration;
   MathMLOperatorDictionary* dictionary;
-  Gtk_MathGraphicDevice* math_graphic_device;
-#if defined(ENABLE_BOXML)
-  Gtk_BoxGraphicDevice* box_graphic_device;
-#endif // ENABLE_BOXML
+  Gtk_Backend* backend;
 };
 
 struct _GtkMathView
@@ -147,10 +153,6 @@ struct _GtkMathView
 
   GtkMathViewModelId current_elem;
 
-  GtkMathViewCursor cursor_visible;
-  GtkMathViewModelId cursor_elem;
-  gint           cursor_index;
-
   MathView*      view;
   Gtk_RenderingContext* renderingContext;
 };
@@ -206,6 +208,8 @@ static guint select_over_signal = 0;
 static guint select_end_signal = 0;
 static guint select_abort_signal = 0;
 static guint element_over_signal = 0;
+static guint decorate_under_signal = 0;
+static guint decorate_over_signal = 0;
 
 /* auxiliary C++ functions */
 
@@ -290,6 +294,8 @@ gtk_math_view_update(GtkMathView* math_view, gint x0, gint y0, gint width, gint
 {
   GtkWidget* widget = GTK_WIDGET(math_view);
 
+  if (!GTK_WIDGET_MAPPED(GTK_WIDGET(math_view)) || math_view->freeze_counter > 0) return;
+
   if (math_view->pixmap != NULL)
     gdk_draw_pixmap(widget->window,
 		    widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
@@ -301,58 +307,7 @@ gtk_math_view_update(GtkMathView* math_view, gint x0, gint y0, gint width, gint
 		       TRUE,
 		       x0, y0, width, height);
 
-  if (math_view->cursor_visible != GTKMATHVIEW_CURSOR_OFF && math_view->cursor_elem != NULL)
-    {
-      GtkMathViewPoint focus_orig;
-      GtkMathViewBoundingBox focus_box;
-      if (!GTKMATHVIEW_METHOD_NAME(get_element_extents)(math_view, math_view->cursor_elem, &focus_orig, &focus_box))
-	return;
-
-      if (math_view->cursor_visible != GTKMATHVIEW_CURSOR_CARET_ON)
-	{
-	  gtk_paint_focus(widget->style,
-			  widget->window,
-			  GTK_STATE_NORMAL,
-			  NULL,
-			  widget,
-			  "?",
-			  focus_orig.x,
-			  focus_orig.y - focus_box.height,
-			  focus_box.width,
-			  focus_box.height + focus_box.depth);
-	}
- 
-      if (math_view->cursor_visible != GTKMATHVIEW_CURSOR_FOCUS_ON && math_view->cursor_index >= 0)
-	{
-	  GdkRectangle crect;
-	  GtkMathViewPoint char_orig;
-	  GtkMathViewBoundingBox char_box;
-	  if (GTKMATHVIEW_METHOD_NAME(get_char_extents)(math_view, math_view->cursor_elem, math_view->cursor_index,
-							&char_orig, &char_box))
-	    crect.x = char_orig.x;
-	  else
-	    {
-	      crect.x = focus_orig.x;
-	      if (math_view->cursor_index > 0) crect.x += focus_box.width;
-	    }
-	  crect.y = focus_orig.y - focus_box.height;
-	  crect.height = focus_box.height + focus_box.depth;
-	  gtk_draw_insertion_cursor(widget,
-				    widget->window,
-				    NULL,
-				    &crect,
-				    TRUE,
-				    GTK_TEXT_DIR_LTR,
-				    FALSE);
-	}
-    }
-}
-
-static void
-gtk_math_view_update(GtkMathView* math_view)
-{
-  GtkWidget* widget = GTK_WIDGET(math_view);
-  gtk_math_view_update(math_view, 0, 0, widget->allocation.width, widget->allocation.height);
+  g_signal_emit(GTK_OBJECT(math_view), decorate_over_signal, 0, widget->window);
 }
 
 static void
@@ -382,20 +337,23 @@ gtk_math_view_paint(GtkMathView* math_view)
     }
 
   rc->setStyle(Gtk_RenderingContext::SELECTED_STYLE);
-  rc->setForegroundColor(math_view_class->configuration->getSelectForeground());
-  rc->setBackgroundColor(math_view_class->configuration->getSelectBackground());
+  rc->setForegroundColor(math_view_class->defaultSelectForeground);
+  rc->setBackgroundColor(math_view_class->defaultSelectBackground);
   rc->setStyle(Gtk_RenderingContext::NORMAL_STYLE);
-  rc->setForegroundColor(math_view_class->configuration->getForeground());
-  rc->setBackgroundColor(math_view_class->configuration->getBackground());
+  rc->setForegroundColor(math_view_class->defaultForeground);
+  rc->setBackgroundColor(math_view_class->defaultBackground);
 
   gdk_draw_rectangle(math_view->pixmap, widget->style->white_gc, TRUE, 0, 0, width, height);
 
+  // WARNING: setAvailableWidth must be invoked BEFORE any coordinate conversion
+  math_view->view->setAvailableWidth(Gtk_RenderingContext::fromGtkX(width));
   gint x = 0;
   gint y = 0;
   to_view_coords(math_view, &x, &y);
+  g_signal_emit(GTK_OBJECT(math_view), decorate_under_signal, 0, math_view->pixmap);
   math_view->view->render(*rc,
-			  Gtk_RenderingContext::fromGtkX(x),
-			  Gtk_RenderingContext::fromGtkY(y));
+			  Gtk_RenderingContext::fromGtkX(-x),
+			  Gtk_RenderingContext::fromGtkY(-y));
 
   gtk_math_view_update(math_view, 0, 0, width, height);
 }
@@ -459,46 +417,6 @@ GTKMATHVIEW_METHOD_NAME(get_type)(void)
   return math_view_type;
 }
 
-static SmartPtr<Configuration>
-initConfiguration(SmartPtr<AbstractLogger>& logger, const char* confPath)
-{
-  SmartPtr<Configuration> configuration = Configuration::create();
-
-  bool res = false;
-  if (confPath != NULL) res = MathView::loadConfiguration(logger, configuration, confPath);
-  if (!res) res = MathView::loadConfiguration(logger, configuration, MathView::getDefaultConfigurationPath());
-  if (!res) res = MathView::loadConfiguration(logger, configuration, "config/gtkmathview.conf.xml");
-  if (!res)
-    {
-      logger->out(LOG_ERROR, "could not load configuration file");
-      exit(-1);
-    }
-
-  return configuration;
-}
-
-static SmartPtr<MathMLOperatorDictionary>
-initOperatorDictionary(const SmartPtr<AbstractLogger>& logger, const SmartPtr<Configuration> configuration)
-{
-  SmartPtr<MathMLOperatorDictionary> dictionary = MathMLOperatorDictionary::create();
-  if (!configuration->getDictionaries().empty())
-    for (std::vector<std::string>::const_iterator dit = configuration->getDictionaries().begin();
-	 dit != configuration->getDictionaries().end();
-	 dit++)
-      {
-	logger->out(LOG_DEBUG, "loading dictionary `%s'", (*dit).c_str());
-	if (!MathView::loadOperatorDictionary(logger, dictionary, (*dit).c_str()))
-	  logger->out(LOG_WARNING, "could not load `%s'", (*dit).c_str());
-      }
-  else
-    {
-      const bool res = MathView::loadOperatorDictionary(logger, dictionary, "config/dictionary.xml");
-      if (!res) MathView::loadOperatorDictionary(logger, dictionary, MathView::getDefaultOperatorDictionaryPath());
-    }
-
-  return dictionary;
-}
-
 static void
 gtk_math_view_base_class_init(GtkMathViewClass* math_view_class)
 {
@@ -508,23 +426,24 @@ gtk_math_view_base_class_init(GtkMathViewClass* math_view_class)
   logger->ref();
   math_view_class->logger = logger;
 
-  SmartPtr<Configuration> configuration = initConfiguration(logger, getenv("GTKMATHVIEWCONF"));
+  SmartPtr<Configuration> configuration = initConfiguration<MathView>(logger, getenv("GTKMATHVIEWCONF"));
   configuration->ref();
   math_view_class->configuration = configuration;
 
-  SmartPtr<MathMLOperatorDictionary> dictionary = initOperatorDictionary(logger, configuration);
+  math_view_class->defaultForeground = configuration->getRGBColor(logger, "default/color/foreground", DEFAULT_FOREGROUND);
+  math_view_class->defaultBackground = configuration->getRGBColor(logger, "default/color/background", DEFAULT_BACKGROUND);
+  math_view_class->defaultSelectForeground = configuration->getRGBColor(logger, "default/select-color/foreground", DEFAULT_SELECT_FOREGROUND);
+  math_view_class->defaultSelectBackground = configuration->getRGBColor(logger, "default/select-color/background", DEFAULT_SELECT_BACKGROUND);
+  math_view_class->defaultT1OpaqueMode = configuration->getBool(logger, "default/t1lib/opaque-mode", false);
+  math_view_class->defaultT1AntiAliasedMode = configuration->getBool(logger, "default/t1lib/anti-aliasing", false);
+
+  SmartPtr<MathMLOperatorDictionary> dictionary = initOperatorDictionary<MathView>(logger, configuration);
   dictionary->ref();
   math_view_class->dictionary = dictionary;
 
-  SmartPtr<Gtk_MathGraphicDevice> mathGraphicDevice = Gtk_MathGraphicDevice::create(logger, configuration);
-  mathGraphicDevice->ref();
-  math_view_class->math_graphic_device = mathGraphicDevice;
-
-#if defined(ENABLE_BOXML)
-  SmartPtr<Gtk_BoxGraphicDevice> boxGraphicDevice = Gtk_BoxGraphicDevice::create();
-  boxGraphicDevice->ref();
-  math_view_class->box_graphic_device = boxGraphicDevice;
-#endif // ENABLE_BOXML
+  SmartPtr<Gtk_Backend> backend = Gtk_Backend::create(logger, configuration);
+  backend->ref();
+  math_view_class->backend = backend;
 }
 
 static void
@@ -532,18 +451,10 @@ gtk_math_view_base_class_finalize(GtkMathViewClass* math_view_class)
 {
   g_return_if_fail(math_view_class != NULL);
 
-#if defined(ENABLE_BOXML)
-  if (math_view_class->box_graphic_device)
+  if (math_view_class->backend)
     {
-      math_view_class->box_graphic_device->unref();
-      math_view_class->box_graphic_device = 0;
-    }
-#endif // ENABLE_BOXML
-
-  if (math_view_class->math_graphic_device)
-    {
-      math_view_class->math_graphic_device->unref();
-      math_view_class->math_graphic_device = 0;
+      math_view_class->backend->unref();
+      math_view_class->backend = 0;
     }
 
   if (math_view_class->dictionary)
@@ -660,7 +571,25 @@ gtk_math_view_class_init(GtkMathViewClass* math_view_class)
 		 G_OBJECT_CLASS_TYPE(object_class),
 		 G_SIGNAL_RUN_FIRST,
 		 G_STRUCT_OFFSET(GtkMathViewClass, element_over),
-		 NULL,NULL,
+		 NULL, NULL,
+		 gtk_marshal_NONE__POINTER,
+		 G_TYPE_NONE, 1, GTK_TYPE_POINTER);
+
+  decorate_under_signal =
+    g_signal_new("decorate_under",
+		 G_OBJECT_CLASS_TYPE(object_class),
+		 G_SIGNAL_RUN_FIRST,
+		 G_STRUCT_OFFSET(GtkMathViewClass, decorate_under),
+		 NULL, NULL,
+		 gtk_marshal_NONE__POINTER,
+		 G_TYPE_NONE, 1, GTK_TYPE_POINTER);
+
+  decorate_over_signal =
+    g_signal_new("decorate_over",
+		 G_OBJECT_CLASS_TYPE(object_class),
+		 G_SIGNAL_RUN_FIRST,
+		 G_STRUCT_OFFSET(GtkMathViewClass, decorate_over),
+		 NULL, NULL, 
 		 gtk_marshal_NONE__POINTER,
 		 G_TYPE_NONE, 1, GTK_TYPE_POINTER);
 
@@ -734,13 +663,6 @@ gtk_math_view_release_document_resources(GtkMathView* math_view)
       g_assert(exc == 0);
       math_view->current_elem = NULL;
     }
-
-  if (math_view->cursor_elem != NULL)
-    {
-      gdome_el_unref(math_view->cursor_elem, &exc);
-      g_assert(exc == 0);
-      math_view->cursor_elem = NULL;
-    }
 }
 
 #else
@@ -749,9 +671,7 @@ static void
 gtk_math_view_release_document_resources(GtkMathView* math_view)
 {
   g_return_if_fail(math_view != NULL);
-
   math_view->current_elem = NULL;
-  math_view->cursor_elem = NULL;
 }
 
 #endif
@@ -768,9 +688,6 @@ gtk_math_view_init(GtkMathView* math_view)
   math_view->select_state    = SELECT_STATE_NO;
   math_view->button_pressed  = FALSE;
   math_view->current_elem    = NULL;
-  math_view->cursor_elem     = NULL;
-  math_view->cursor_index    = 0;
-  math_view->cursor_visible  = GTKMATHVIEW_CURSOR_OFF;
   math_view->hadjustment = NULL;
   math_view->vadjustment = NULL;
   math_view->top_x = math_view->top_y = 0;
@@ -786,13 +703,15 @@ gtk_math_view_init(GtkMathView* math_view)
 
   view->setLogger(math_view_class->logger);
   view->setOperatorDictionary(math_view_class->dictionary);
-  view->setMathMLNamespaceContext(MathMLNamespaceContext::create(view, math_view_class->math_graphic_device));
+  view->setMathMLNamespaceContext(MathMLNamespaceContext::create(view, math_view_class->backend->getMathGraphicDevice()));
 #if ENABLE_BOXML
-  view->setBoxMLNamespaceContext(BoxMLNamespaceContext::create(view, math_view_class->box_graphic_device));
+  view->setBoxMLNamespaceContext(BoxMLNamespaceContext::create(view, math_view_class->backend->getBoxGraphicDevice()));
 #endif // ENABLE_BOXML
 
   math_view->renderingContext = new Gtk_RenderingContext(math_view_class->logger);
   math_view->renderingContext->setColorMap(gtk_widget_get_colormap(GTK_WIDGET(math_view)));
+  math_view->renderingContext->setT1OpaqueMode(math_view_class->defaultT1OpaqueMode);
+  math_view->renderingContext->setT1AntiAliasedMode(math_view_class->defaultT1AntiAliasedMode);
 #endif
 }
 
@@ -853,7 +772,6 @@ gtk_math_view_destroy(GtkObject* object)
     (*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
 }
 
-
 extern "C" gboolean
 GTKMATHVIEW_METHOD_NAME(freeze)(GtkMathView* math_view)
 {
@@ -875,6 +793,16 @@ GTKMATHVIEW_METHOD_NAME(thaw)(GtkMathView* math_view)
     return FALSE;
 }
 
+extern "C" void
+GTKMATHVIEW_METHOD_NAME(update)(GtkMathView* math_view, GdkRectangle* rect)
+{
+  GtkWidget* widget = GTK_WIDGET(math_view);
+  if (rect)
+    gtk_math_view_update(math_view, rect->x, rect->y, rect->width, rect->height);
+  else
+    gtk_math_view_update(math_view, 0, 0, widget->allocation.width, widget->allocation.height);
+}
+
 static void
 gtk_math_view_realize(GtkWidget* widget)
 {
@@ -911,8 +839,6 @@ gtk_math_view_realize(GtkWidget* widget)
   gtk_style_set_background (widget->style, widget->window, GTK_STATE_ACTIVE);
 }
 
-#include <iostream>
-
 static void
 gtk_math_view_size_request(GtkWidget* widget, GtkRequisition* requisition)
 {
@@ -1001,7 +927,7 @@ gtk_math_view_button_release_event(GtkWidget* widget, GdkEventButton* event)
 #endif
       GtkMathViewModelId elem = NULL;
 
-      gtk_math_view_get_element_at(math_view, (gint) event->x, (gint) event->y, &elem, NULL, NULL);
+      GTKMATHVIEW_METHOD_NAME(get_element_at)(math_view, (gint) event->x, (gint) event->y, &elem, NULL, NULL);
 
       GtkMathViewModelEvent me;
       me.id = elem;
@@ -1133,7 +1059,7 @@ gtk_math_view_expose_event(GtkWidget* widget,
   if (math_view->pixmap == NULL)
     gtk_math_view_paint(math_view);
   else
-    gtk_math_view_update(math_view, event->area.x, event->area.y, event->area.width, event->area.height);
+    GTKMATHVIEW_METHOD_NAME(update)(math_view, event->area.x, event->area.y, event->area.width, event->area.height);
 
   return FALSE;
 }
@@ -1371,7 +1297,7 @@ extern "C" gboolean
 GTKMATHVIEW_METHOD_NAME(load_reader)(GtkMathView* math_view, xmlTextReaderPtr reader)
 {
   g_return_val_if_fail(math_view != NULL, FALSE);
-  g_return_val_if_fail(math_view->view != NULL, NULL);
+  g_return_val_if_fail(math_view->view != NULL, FALSE);
   g_return_val_if_fail(reader != NULL, FALSE);
 
   gtk_math_view_release_document_resources(math_view);
@@ -1650,18 +1576,27 @@ GTKMATHVIEW_METHOD_NAME(get_element_at)(GtkMathView* math_view, gint x, gint y,
 }
 
 extern "C" gboolean
-GTKMATHVIEW_METHOD_NAME(get_element_extents)(GtkMathView* math_view, GtkMathViewModelId el,
-					     GtkMathViewPoint* result_orig, GtkMathViewBoundingBox* result_box)
+GTKMATHVIEW_METHOD_NAME(get_element_extents_ref)(GtkMathView* math_view, GtkMathViewModelId refEl, GtkMathViewModelId el,
+						 GtkMathViewPoint* result_orig, GtkMathViewBoundingBox* result_box)
 {
   g_return_val_if_fail(math_view != NULL, FALSE);
   g_return_val_if_fail(math_view->view != NULL, FALSE);
   g_return_val_if_fail(el != NULL, FALSE);
 
+  SmartPtr<Element> refElem;
+  if (refEl)
+    {
+      refElem = math_view->view->elementOfModelElement(refEl);
+      if (!refElem) return FALSE;
+    }
+
   if (SmartPtr<Element> elem = math_view->view->elementOfModelElement(el))
     {
       Point elemOrig;
       BoundingBox elemBox;
-      if (math_view->view->getElementExtents(elem, result_orig ? &elemOrig : 0, result_box ? &elemBox : 0))
+
+      if ((refElem && math_view->view->getElementExtents(refElem, elem, result_orig ? &elemOrig : 0, result_box ? &elemBox : 0))
+	  || math_view->view->getElementExtents(elem, result_orig ? &elemOrig : 0, result_box ? &elemBox : 0))
 	{
 	  if (result_orig)
 	    {
@@ -1683,6 +1618,13 @@ GTKMATHVIEW_METHOD_NAME(get_element_extents)(GtkMathView* math_view, GtkMathView
   return FALSE;
 }
 
+extern "C" gboolean
+GTKMATHVIEW_METHOD_NAME(get_element_extents)(GtkMathView* math_view, GtkMathViewModelId el,
+					     GtkMathViewPoint* result_orig, GtkMathViewBoundingBox* result_box)
+{
+  return GTKMATHVIEW_METHOD_NAME(get_element_extents_ref)(math_view, NULL, el, result_orig, result_box);
+}
+
 extern "C" gboolean
 GTKMATHVIEW_METHOD_NAME(get_char_at)(GtkMathView* math_view, gint x, gint y,
 				     GtkMathViewModelId* result_elem, gint* result_index,
@@ -1727,19 +1669,27 @@ GTKMATHVIEW_METHOD_NAME(get_char_at)(GtkMathView* math_view, gint x, gint y,
 }
 
 extern "C" gboolean
-GTKMATHVIEW_METHOD_NAME(get_char_extents)(GtkMathView* math_view, GtkMathViewModelId el,
-					  gint index, GtkMathViewPoint* result_orig, GtkMathViewBoundingBox* result_box)
+GTKMATHVIEW_METHOD_NAME(get_char_extents_ref)(GtkMathView* math_view, GtkMathViewModelId refEl, GtkMathViewModelId el,
+					      gint index, GtkMathViewPoint* result_orig, GtkMathViewBoundingBox* result_box)
 {
   g_return_val_if_fail(math_view != NULL, FALSE);
   g_return_val_if_fail(math_view->view != NULL, FALSE);
   g_return_val_if_fail(el != NULL, FALSE);
   g_return_val_if_fail(index >= 0, FALSE);
 
+  SmartPtr<Element> refElem;
+  if (refEl)
+    {
+      refElem = math_view->view->elementOfModelElement(refEl);
+      if (!refElem) return FALSE;
+    }
+
   if (SmartPtr<Element> elem = math_view->view->elementOfModelElement(el))
     {
       Point charOrig;
       BoundingBox charBox;
-      if (math_view->view->getCharExtents(elem, index, result_orig ? &charOrig : 0, result_box ? &charBox : 0))
+      if ((refElem && math_view->view->getCharExtents(refElem, elem, index, result_orig ? &charOrig : 0, result_box ? &charBox : 0))
+	  || math_view->view->getCharExtents(elem, index, result_orig ? &charOrig : 0, result_box ? &charBox : 0))
 	{
 	  if (result_orig)
 	    {
@@ -1762,70 +1712,11 @@ GTKMATHVIEW_METHOD_NAME(get_char_extents)(GtkMathView* math_view, GtkMathViewMod
   return FALSE;
 }
 
-extern "C" void
-GTKMATHVIEW_METHOD_NAME(get_cursor)(GtkMathView* math_view, GtkMathViewModelId* elem, gint* index)
+extern "C" gboolean
+GTKMATHVIEW_METHOD_NAME(get_char_extents)(GtkMathView* math_view, GtkMathViewModelId el,
+					  gint index, GtkMathViewPoint* result_orig, GtkMathViewBoundingBox* result_box)
 {
-  g_return_if_fail(math_view != NULL);
-  if (elem != NULL)
-    {
-      *elem = math_view->cursor_elem;
-#if GTKMATHVIEW_USES_GMETADOM
-      if (*elem)
-	{
-	  GdomeException exc = 0;
-	  gdome_el_ref(*elem, &exc);
-	  g_assert(exc == 0);
-	}
-#endif // GTKMATHVIEW_USES_GMETADOM
-    }
-
-  if (index != NULL) *index = math_view->cursor_index;
-}
-
-extern "C" void
-GTKMATHVIEW_METHOD_NAME(set_cursor)(GtkMathView* math_view, GtkMathViewModelId elem, gint index)
-{
-  g_return_if_fail(math_view != NULL);
-  if (math_view->cursor_elem != elem || math_view->cursor_index != index)
-    {
-      math_view->cursor_index = index;
-#if GTKMATHVIEW_USES_GMETADOM
-      if (math_view->cursor_elem != NULL)
-	{
-	  GdomeException exc = 0;
-	  gdome_el_unref(math_view->cursor_elem, &exc);
-	  g_assert(exc == 0);
-	}
-#endif // GTKMATHVIEW_USES_GMETADOM
-      math_view->cursor_elem = elem;
-#if GTKMATHVIEW_USES_GMETADOM
-      if (math_view->cursor_elem != NULL)
-	{
-	  GdomeException exc = 0;
-	  gdome_el_ref(math_view->cursor_elem, &exc);
-	  g_assert(exc == 0);
-	}
-#endif // GTKMATHVIEW_USES_GMETADOM
-      if (math_view->cursor_visible) gtk_math_view_update(math_view);
-    }
-}
-
-extern "C" void
-GTKMATHVIEW_METHOD_NAME(set_cursor_visible)(GtkMathView* math_view, GtkMathViewCursor visible)
-{
-  g_return_if_fail(math_view != NULL);
-  if (math_view->cursor_visible != visible)
-    {
-      math_view->cursor_visible = visible;
-      if (math_view->cursor_visible) gtk_math_view_update(math_view);
-    }
-}
-
-extern "C" GtkMathViewCursor
-GTKMATHVIEW_METHOD_NAME(get_cursor_visible)(GtkMathView* math_view)
-{
-  g_return_val_if_fail(math_view != NULL, GTKMATHVIEW_CURSOR_OFF);
-  return math_view->cursor_visible;
+  return GTKMATHVIEW_METHOD_NAME(get_char_extents_ref)(math_view, NULL, el, index, result_orig, result_box);
 }
 
 extern "C" void
