@@ -25,9 +25,7 @@
 #ifndef __gtkmathview_common_h__
 #define __gtkmathview_common_h__
 
-#include <gtk/gtkframe.h>
-#include <gtk/gtkeventbox.h>
-#include <gtk/gtkdrawingarea.h>
+#include <gtk/gtkwidget.h>
 
 #ifdef __cplusplus
 extern "C" {
@@ -75,13 +73,6 @@ extern "C" {
   typedef GdomeDOMString*           GtkMathViewModelString;
 #endif
 
-  typedef enum _GtkMathViewCursor {
-    GTKMATHVIEW_CURSOR_OFF,
-    GTKMATHVIEW_CURSOR_CARET_ON,
-    GTKMATHVIEW_CURSOR_FOCUS_ON,
-    GTKMATHVIEW_CURSOR_ON
-  } GtkMathViewCursor;
-
   typedef struct _GtkMathViewModelEvent {
     GtkMathViewModelId id;
     gint x;
@@ -91,11 +82,13 @@ extern "C" {
 
   typedef void (*GtkMathViewModelSignal)(GtkMathView*, const GtkMathViewModelEvent*);
   typedef void (*GtkMathViewSelectAbortSignal)(GtkMathView*);
+  typedef void (*GtkMathViewDecorateSignal)(GtkMathView*, GdkDrawable*, gpointer);
 
   GtkType    GTKMATHVIEW_METHOD_NAME(get_type)(void);
   GtkWidget* GTKMATHVIEW_METHOD_NAME(new)(GtkAdjustment*, GtkAdjustment*);
   gboolean   GTKMATHVIEW_METHOD_NAME(freeze)(GtkMathView*);
   gboolean   GTKMATHVIEW_METHOD_NAME(thaw)(GtkMathView*);
+  void       GTKMATHVIEW_METHOD_NAME(update)(GtkMathView*, GdkRectangle*);
 #if GTKMATHVIEW_USES_CUSTOM_READER
   gboolean   GTKMATHVIEW_METHOD_NAME(load_reader)(GtkMathView*, GtkMathViewReader*, GtkMathViewReaderData);
 #elif GTKMATHVIEW_USES_LIBXML2_READER
@@ -118,14 +111,16 @@ extern "C" {
 						     GtkMathViewPoint*, GtkMathViewBoundingBox*);
   gboolean   GTKMATHVIEW_METHOD_NAME(get_element_extents)(GtkMathView*, GtkMathViewModelId,
 							  GtkMathViewPoint*, GtkMathViewBoundingBox*);
+  gboolean   GTKMATHVIEW_METHOD_NAME(get_element_extents_ref)(GtkMathView*, GtkMathViewModelId,
+							      GtkMathViewModelId,
+							      GtkMathViewPoint*, GtkMathViewBoundingBox*);
   gboolean   GTKMATHVIEW_METHOD_NAME(get_char_at)(GtkMathView*, gint, gint, GtkMathViewModelId*, gint*,
 						  GtkMathViewPoint*, GtkMathViewBoundingBox*);
   gboolean   GTKMATHVIEW_METHOD_NAME(get_char_extents)(GtkMathView*, GtkMathViewModelId, gint,
 						       GtkMathViewPoint*, GtkMathViewBoundingBox*);
-  void       GTKMATHVIEW_METHOD_NAME(set_cursor)(GtkMathView*, GtkMathViewModelId, gint);
-  void       GTKMATHVIEW_METHOD_NAME(get_cursor)(GtkMathView*, GtkMathViewModelId*, gint*);
-  void       GTKMATHVIEW_METHOD_NAME(set_cursor_visible)(GtkMathView*, GtkMathViewCursor);
-  GtkMathViewCursor GTKMATHVIEW_METHOD_NAME(get_cursor_visible)(GtkMathView*);
+  gboolean   GTKMATHVIEW_METHOD_NAME(get_char_extents_ref)(GtkMathView*,
+							   GtkMathViewModelId, GtkMathViewModelId, gint,
+							   GtkMathViewPoint*, GtkMathViewBoundingBox*);
   void       GTKMATHVIEW_METHOD_NAME(get_size)(GtkMathView*, gint*, gint*);
   void       GTKMATHVIEW_METHOD_NAME(get_top)(GtkMathView*, gint*, gint*);
   void       GTKMATHVIEW_METHOD_NAME(set_top)(GtkMathView*, gint, gint);
