@@ -23,6 +23,8 @@
  */
 
 #include <config.h>
+
+#include "defs.h"
 #include <stdio.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
@@ -36,8 +38,6 @@ extern GdomeElement* find_xref_element(GdomeElement*);
 extern GdomeElement* find_common_ancestor(GdomeElement*, GdomeElement*);
 extern GdomeElement* find_self_or_ancestor(GdomeElement*, const char*, const char*);
 
-#define XLINK_NS_URI "http://www.w3.org/1999/xlink"
-
 static GtkWidget* window;
 static GtkWidget* main_area;
 static GtkWidget* scrolled_area;
@@ -49,6 +49,7 @@ static gboolean   semantic_selection = FALSE;
 static gchar* doc_name = NULL;
 static GdomeElement* first_selected = NULL;
 static GdomeElement* root_selected = NULL;
+static GtkMathViewDefaultCursorDecorator* cursor = NULL;
 
 static guint statusbar_context;
 
@@ -477,13 +478,15 @@ element_over(GtkMathView* math_view, const GtkMathViewModelEvent* event)
     gdome_str_unref(link);
 
   action = find_action_element(event->id);
-  if (action != NULL)
+#if 0
+  if (action != NULL) 
     {
-      gtk_math_view_set_cursor(math_view, action, -1);
-      gtk_math_view_set_cursor_visible(math_view, GTKMATHVIEW_CURSOR_ON);
+      GdomeException exc = 0;
+      gtk_math_view_decor_default_cursor_set(cursor, TRUE, action, TRUE, -1, FALSE);
+      gdome_el_unref(action, &exc);
+      g_assert(exc == 0);
     }
-  else
-    gtk_math_view_set_cursor(math_view, NULL, -1);
+#endif
 }
 
 static void
@@ -642,19 +645,12 @@ click(GtkMathView* math_view, const GtkMathViewModelEvent* event)
   g_return_if_fail(math_view != NULL);
   g_return_if_fail(event != NULL);
 
-#if 0
+#if 1
   index = -1;
   if (gtk_math_view_get_char_at(math_view, event->x, event->y, &elem, &index, NULL, NULL))
-    {
-      /*printf("get_char_at: (event->id = %p, elem = %p) %d\n", event->id, elem, index);*/
-      gtk_math_view_set_cursor(math_view, event->id, index);
-      gtk_math_view_set_cursor_visible(math_view, GTKMATHVIEW_CURSOR_ON);
-    }
+    gtk_math_view_decor_default_cursor_set(cursor, TRUE, event->id, TRUE, index, TRUE);
   else
-    {
-      /*printf("get_char_at: failed\n");*/
-      gtk_math_view_set_cursor_visible(math_view, GTKMATHVIEW_CURSOR_OFF);
-    }
+    gtk_math_view_decor_default_cursor_set(cursor, FALSE, NULL, FALSE, -1, FALSE);
 #endif
 
   if (event->id != NULL)
@@ -697,8 +693,25 @@ click(GtkMathView* math_view, const GtkMathViewModelEvent* event)
       root_selected = NULL;
     }
 }
+
 #endif // HAVE_GMETADOM
 
+static gboolean
+cursor_blink(GtkMathViewDefaultCursorDecorator* cursor)
+{
+  gboolean enabled;
+  GdomeElement* focus;
+  gboolean draw_focus;
+  gint index;
+  gboolean char_index;
+  g_return_val_if_fail(cursor != NULL, FALSE);
+
+  gtk_math_view_decor_default_cursor_get(cursor, &enabled, &focus, &draw_focus, &index, &char_index);
+  gtk_math_view_decor_default_cursor_set(cursor, !enabled, focus, draw_focus, index, char_index);
+
+  return TRUE;
+}
+
 static void
 create_widget_set()
 {
@@ -757,6 +770,9 @@ create_widget_set()
 		   G_CALLBACK(click),
 		   (gpointer) main_area);
 
+  cursor = gtk_math_view_decor_default_cursor_new(main_area);
+  g_timeout_add(500, cursor_blink, cursor);
+
   status_bar = gtk_statusbar_new();
   gtk_widget_show(status_bar);
   gtk_box_pack_start(GTK_BOX(main_vbox), status_bar, FALSE, TRUE, 0);
